/*
** Module   : MenuWidget.java
** Abstract : server side menu widget implementation
**
** Copyright (c) 2014-2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- ----------------------------Description-----------------------------
** 001 VIG 20141029          Created initial version.
*/
package com.goldencode.p2j.ui;

import java.lang.reflect.*;
import java.util.*;

import com.goldencode.p2j.util.*;
import com.goldencode.proxy.*;

/**
 * This class represents menu widget.
 */
@LegacyResource(resource = LegacyResource.MENU)
public class MenuWidget
extends PaneEntity<MenuConfig>
{
   /**
    * Default constructor.  
    * 
    * @param    dynamic
    *           Flag indicating if this is a static or dynamic resource.
    */
   public MenuWidget(boolean dynamic)
   {
      super(dynamic, new MenuConfig());
   }

   /**
    * Creates new menu as a proxy object (as a local resource).  This loads
    * the static menu definition.
    *
    * @param    <T> 
    *           Frame interface type.
    * @param    iface
    *           The class of the interface definition. 
    * @param    name
    *           The frame name.
    *
    * @return   The created frame.
    */
   public static <T extends MenuDefinition> MenuWidget createStaticMenu(Class<T> definition)
   {
      MenuWidget menu = new MenuWidget(false);
      
      List<Class<?>> ifaceList = new ArrayList<Class<?>>();
      
      // proxy the interface and all of its ancestor superinterfaces, except
      // MenuDefinition
      Class<?> next = definition;
      while (next != null && !MenuDefinition.class.equals(next))
      {
         ifaceList.add(next);
         Class<?>[] ext = next.getInterfaces();
         next = (ext.length == 1 ? ext[0] : null);
      }
      
      // also proxy InvocationHandler
      //ifaceList.add(InvocationHandler.class);
      
      Class<?>[] ifaces = new Class<?>[ifaceList.size()];
      ifaces = ifaceList.toArray(ifaces);
      
      // manufacture an instance that will redirect method calls to our
      // invocation handler

      T proxy = (T) ProxyFactory.getProxy(MenuDefinition.class, ifaces);
      proxy.setup(menu);
      return menu;
   }
}
